#import "PrinterInfoController.h"
#import "ePOS-Print.h"
#import <sys/time.h>

static unsigned char CMD_ESDPOS[] = {
    0x1b, 0x3d, 0x01,    // ESC = 1(Enables printer)
    0x1d, 0x49, 0x45,    // GS I 69(Type of mounted additional fonts)
    0x1d, 0x49, 0x43,    // GS I 67(Printer name)
};
#define RESPONSE_HEADER       0x5f
#define RESPONSE_TERMINAL     0x00
#define SEND_RESPONSE_TIMEOUT 1000
#define RESPONSE_MAXBYTE      128

@interface PrinterInfoController ()
- (long long)currentTimeMillis;
- (int)receiveResponse:(EpsonIo*)port;
- (BOOL)analyzeResponse:(NSData*)receiveData length:(int)bufferlen;
@end

@implementation PrinterInfoController


- (void)dealloc
{
}

- (BOOL)getPrinterNameWithDeviceName:(NSString *)deviceName connectionType:(int)connectionType
{
    //open
    EpsonIo *port = [[EpsonIo alloc] init];
    if(port == nil){
        return NO;
    }
    
    // parse device Type
    int epsonIOConnectionType = [self parseDevTypeFromEposPrintToEpsonIo:connectionType];
    int result = [port open:epsonIOConnectionType DeviceName:deviceName DeviceSettings:nil];
    if(result != EPSONIO_OC_SUCCESS){
        port = nil;
        return NO;
    }
        
    //send command(esc/pos)
    size_t written = 0;
    NSData * escpos = [[NSData alloc] initWithBytes:CMD_ESDPOS length:sizeof(CMD_ESDPOS)];
    result = [port write:escpos Offset:0 Size:[escpos length] Timeout:SEND_RESPONSE_TIMEOUT SizeWritten:&written];
    if(result != EPSONIO_OC_SUCCESS){
        [port close];
        port = nil;
        return NO;
    }

    //receive response
    result = [self receiveResponse:port];
    if(result != EPSONIO_OC_SUCCESS){
        [port close];
        port = nil;
        return NO;
    }
       
    //close
    [port close];
    port = nil;
    
    return YES;
}

- (long long)currentTimeMillis
{
    struct timeval ts;
    gettimeofday(&ts, NULL);
    return (unsigned long long)ts.tv_sec * 1000 + ts.tv_usec / 1000;
}

//receive response
- (int)receiveResponse:(EpsonIo*)port
{
    NSMutableData * receiveBuffer = [[NSMutableData alloc] initWithLength:RESPONSE_MAXBYTE];
    int receiveCurrent = 0;
    
    //receive loop(timeout=1000ms)
    long long starttime = [self currentTimeMillis];
    while(true){
        //check receive buffer full
        if(receiveCurrent >= [receiveBuffer length]){
            break;
        }
        
        //timeout check
        long long readTimeout = SEND_RESPONSE_TIMEOUT - ([self currentTimeMillis] - starttime);
        if(readTimeout < 0){
            break;
        }
        
        //receive
        size_t sizeRead;
        int result = [port read:receiveBuffer
                         Offset:receiveCurrent
                           Size:receiveBuffer.length - receiveCurrent
                        Timeout:(long)readTimeout
                       SizeRead:&sizeRead];
        if(result == EPSONIO_OC_ERR_TIMEOUT){
            break;
        }else if(result != EPSONIO_OC_SUCCESS){
            return result;
        }
        receiveCurrent += sizeRead;
        
        //analyze receive data
        if([self analyzeResponse:receiveBuffer length:receiveCurrent]){
            break;
        }
    }
    
    return EPSONIO_OC_SUCCESS;
}

//analyze GS I response
- (BOOL)analyzeResponse:(NSData*)receiveData length:(int)bufferlen
{
    int currentPos = 0;
    NSString* responseFontName = nil;
    NSString* responsePrinterName = nil;
    
    const unsigned char *buffer = [receiveData bytes];
    //search 5f header
    for(; currentPos < bufferlen; currentPos++){
        if(buffer[currentPos] == RESPONSE_HEADER){
            break;
        }
    }
    if(currentPos == bufferlen){
        return false;
    }
    while(responseFontName == nil || responsePrinterName == nil){
        //check 5f header
        if(buffer[currentPos] != RESPONSE_HEADER){
            return false;
        }
        
        //find terminal(0x00)
        int endPos = currentPos + 1;
        for(; endPos < bufferlen; endPos++){
            if(buffer[endPos] == RESPONSE_TERMINAL){
                break;
            }
        }
        if(endPos >= bufferlen){
            return false;
        }
        
        //get response string
        NSString *responseString
            = [[NSString alloc] initWithBytes:buffer + currentPos + 1
                                       length:endPos - currentPos - 1
                                     encoding:NSASCIIStringEncoding];
        if(responseFontName == nil){
            responseFontName = responseString;
        }else{
            responsePrinterName = responseString;
        }
        
        currentPos = endPos + 1;
    }
    
    fontName_ = [responseFontName stringByTrimmingCharactersInSet:
                  [NSCharacterSet whitespaceCharacterSet]] ;
    printerName_ = [responsePrinterName stringByTrimmingCharactersInSet:
                  [NSCharacterSet whitespaceCharacterSet]];
    return true;
}

//----------------------------------------------------------------------------//
#pragma mark - Get Infomation Method
//----------------------------------------------------------------------------//
- (NSString*)printerModel
{
    return printerName_;
}

- (int)language
{
    if(!fontName_){
        return EPOS_OC_MODEL_ANK;
    }
    
    if([fontName_ isEqualToString:@""]){
        return EPOS_OC_MODEL_ANK;
    }else if([fontName_ isEqualToString:@"KANJI JAPANESE"]){
        return EPOS_OC_MODEL_JAPANESE;
    }else{
        return EPOS_OC_MODEL_ANK;
    }
}

- (int)parseDevTypeFromEposPrintToEpsonIo:(int)eposPrintDevType{
    
    switch(eposPrintDevType){
        case EPOS_OC_DEVTYPE_TCP:           return EPSONIO_OC_DEVTYPE_TCP;
        case EPOS_OC_DEVTYPE_BLUETOOTH:     return EPSONIO_OC_DEVTYPE_BLUETOOTH;
        default:                            return EPSONIO_OC_DEVTYPE_TCP;
    }
}

@end
