#import "PrintQrCodeTask.h"
#import "ePOS-Print.h"
#import "ePOSEasySelect.h"

static const long PRINT_QRCODE_TIMEOUT  = 5000;

static NSString* const  PRINT_TEXT_DEVICE       = @"Device:";
static NSString* const  PRINT_TEXT_INTERFACE    = @"Interface:";
static NSString* const  PRINT_TEXT_ADDRESS      = @"Address:";
static NSString* const  PRINT_TEXT_WIFI         = @"Wi-Fi";
static NSString* const  PRINT_TEXT_BT           = @"Bluetooth";

@implementation PrintQrCodeTask
{
    NSString* printerModel_;
    NSString* address_;
    NSString* MACAddress_;
    int connectionType_;
    int lang_;
}

- (instancetype)init
{
    @throw [NSException exceptionWithName:@"Don't use init"
                                   reason:@"Please use designated initializer"
                                 userInfo:nil];
    return nil;
}

// Designated initializer
- (instancetype)initWithPrinterModel:(NSString*)printerModel address:(NSString*)address MACAddress:(NSString*)MACAddress connectionType:(const int)connectionType lang:(const int)lang
{
    self = [super init];
    if(self){
        printerModel_ = printerModel;
        address_ = address;
        MACAddress_ = MACAddress;
        connectionType_ = connectionType;
        lang_ = lang;
    }
    return self;
}

//----------------------------------------------------------------------------//
#pragma mark - Print Method
//----------------------------------------------------------------------------//
- (BOOL)printQr:(long)qrCodeSize
{
    // Create builder
    EposBuilder *builder = [[EposBuilder alloc] initWithPrinterModel:printerModel_ Lang:lang_];
    if(!builder){
        return NO;
    }
    
    // Add data to builder
    BOOL isSuccess = [self makeHeader:builder];
    if(!isSuccess){
        return NO;
    }
    
    isSuccess = [self makeQrCode:builder qrCodeSize:qrCodeSize];
    if(!isSuccess){
        return NO;
    }
    
    isSuccess = [self makeFooter:builder];
    if(!isSuccess){
        return NO;
    }
    
    // Create epos print object
    EposPrint *eposPrint = [[EposPrint alloc] init];
    if(!eposPrint){
        return NO;
    }
    
    // Open printer
    int result = [eposPrint openPrinter:connectionType_ DeviceName:address_];
    if(result != EPOS_OC_SUCCESS){
        return NO;
    }

    // Send data
    unsigned long status = 0;
    result = [eposPrint sendData:builder
                         Timeout:PRINT_QRCODE_TIMEOUT
                          Status:&status];
    
    // Close printer
    [eposPrint closePrinter];
    
    if(result == EPOS_OC_SUCCESS){
        return YES;
    } else {
        return NO;
    }
}

//----------------------------------------------------------------------------//
#pragma mark - Make Header
//----------------------------------------------------------------------------//
- (BOOL)makeHeader:(EposBuilder*)builder
{
    if(!builder){
        return NO;
    }
    
    // Device Name
    int result = [builder addText:PRINT_TEXT_DEVICE];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }

    result = [builder addText:printerModel_];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }

    result = [builder addFeedLine:1];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    // Interface
    result = [builder addText:PRINT_TEXT_INTERFACE];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
        
    NSString* interfaceText = [self interfaceText];
    result = [builder addText:interfaceText];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:1];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    // Mac Address
    result = [builder addText:PRINT_TEXT_ADDRESS];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addText:MACAddress_];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addFeedLine:2];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    return YES;
}

//----------------------------------------------------------------------------//
#pragma mark - Make QR CODE
//----------------------------------------------------------------------------//
- (BOOL)makeQrCode:(EposBuilder*)builder qrCodeSize:(long)qrCodeSize
{
    if(!builder){
        return NO;
    }

    EposEasySelect* easySelect = [[EposEasySelect alloc] init];
    NSString* qrCodeText = [easySelect createQR:printerModel_
                                     DeviceType:connectionType_
                                     MacAddress:MACAddress_];
    
    if(!qrCodeText){
        return NO;
    }
    
    // Add QR CODE
    int result = [builder addTextAlign:EPOS_OC_ALIGN_CENTER];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    
    result = [builder addSymbol:qrCodeText
                           Type:EPOS_OC_SYMBOL_QRCODE_MODEL_2
                          Level:EPOS_OC_LEVEL_L
                          Width:qrCodeSize      // Specifies the QR code width
                         Height:qrCodeSize      // Ignored by QR code
                           Size:0];             // Ignored by QR code
    if(result != EPOS_OC_SUCCESS){
        return NO;
    }
    
    return YES;
}

//----------------------------------------------------------------------------//
#pragma mark - Make Footer (Feed & CUT)
//----------------------------------------------------------------------------//
- (BOOL)makeFooter:(EposBuilder*)builder
{
    int result = [builder addFeedLine:1];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }

    result = [builder addCut:EPOS_OC_CUT_FEED];
    if (EPOS_OC_SUCCESS != result) {
        return NO;
    }
    return YES;
}

//----------------------------------------------------------------------------//
#pragma mark - Print QR CODE Utility Method
//----------------------------------------------------------------------------//
- (NSString*)interfaceText
{
    switch(connectionType_){
        case EPOS_OC_DEVTYPE_BLUETOOTH:
            return PRINT_TEXT_BT;
        case EPOS_OC_DEVTYPE_TCP:
            return PRINT_TEXT_WIFI;
        default:
            return PRINT_TEXT_WIFI;
    }
}
@end
