#import <AVFoundation/AVFoundation.h>
#import <QRCodeCapture.h>
#import "QuickParingViewController.h"
#import "IndicatorView.h"
#import "ePOSEasySelect.h"
#import "ePOS-Print.h"
#import "PrintSampleTask.h"
#import "Utility.h"

typedef NS_ENUM(NSInteger, QuickParingError) {
    QuickParingErrorOpen = 0,
    QuickParingErrorPrint,
};

@interface QuickParingViewController () <QRCodeCaptureDelegate, UIAlertViewDelegate>
{
    IndicatorView* qpIndicatorView_;
    EposEasySelectInfo* easySelectInfo_;
    QRCodeCapture* qpQrCodeCapture_;        //  using libzxing
}
@end

@implementation QuickParingViewController


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    if ([self respondsToSelector:@selector(edgesForExtendedLayout)]){
        self.edgesForExtendedLayout = UIRectEdgeNone;
    }
    
    qpQrCodeCapture_ = [[QRCodeCapture alloc] init];
    BOOL isConnectedSuccess = [qpQrCodeCapture_ connectCaptureSettion];
    if(isConnectedSuccess){
        _qpCannotUseVideoUILabel.hidden = YES;
    } else {
        qpQrCodeCapture_ = nil;
    }
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(willEnterForeground) name:UIApplicationWillEnterForegroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didEnterBackground) name:UIApplicationDidEnterBackgroundNotification object:nil];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    self.title = NSLocalizedString(@"QP_Title_QuickPairingSample", @"");
    
    [self initializeTargetInfo];
    [Utility UIButtonDisableWithColor:self.printUIButton];
    
    self.qpInfoBackgroundUIView.layer.cornerRadius = 5;
    self.qpInfoBackgroundUIView.clipsToBounds = true;
    
    [self resizeQPPreviewTargetView];   // Resize uiview
    
    [qpQrCodeCapture_ startCapture:self.qpPreviewTargetUIView scanDelegate:self];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    [qpQrCodeCapture_ stopCapture];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    easySelectInfo_ = nil;
    [qpQrCodeCapture_ disConnectCaptureSettion];
}
//----------------------------------------------------------------------------//
#pragma mark - QRCodeCaptureDelegate
//----------------------------------------------------------------------------//
- (void)didScanResult:(NSString *)result
{
    qpQrCodeCapture_.delegate = nil;

    // Analyze QR code
    EposEasySelect* easySelect = [[EposEasySelect alloc] init];
    EposEasySelectInfo *easySelectInfo = [easySelect parseQR:result];
    
    if(easySelectInfo){
        
        [qpQrCodeCapture_ setFrameColor:[UIColor greenColor]];
        [[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.05]];   // for update display

        [self updateTargetInfo:easySelectInfo];
        _connectingUILabel.text = NSLocalizedString(@"QP_Lbl_Connecting", @"");
        [qpQrCodeCapture_ stopCapture];
        [self showIndicator];

        BOOL printerAlive = [self connectPrinter:easySelectInfo];
        
        [self hideIndicator];
        _connectingUILabel.text = @"";
        
        if (printerAlive) {
            [Utility UIButtonEnableWithColor:self.printUIButton];
            [qpQrCodeCapture_ startCapture:self.qpPreviewTargetUIView scanDelegate:self];

        }else {
            [Utility UIButtonDisableWithColor:self.printUIButton];
            [self showQpErrorMsg:QuickParingErrorOpen];
            // Start capturing after alert ok button clicked (-alertView:clickedButtonAtIndex:)
        }
        
    } else {
        qpQrCodeCapture_.delegate = self;
    }
}

//----------------------------------------------------------------------------//
#pragma mark - EposPrint Management Method
//----------------------------------------------------------------------------//
- (BOOL)connectPrinter:(EposEasySelectInfo*)easySelectInfo
{
    // Connect printer
    EposPrint *eposPrint = [[EposPrint alloc] init];
    if(!eposPrint){
        return NO;
    }
    
    int result = [eposPrint openPrinter:easySelectInfo.deviceType DeviceName:easySelectInfo.macAddress];

	//
	// If you want to print immediately, send print command here.
	//

    [eposPrint closePrinter];
    
    if(result == EPOS_OC_SUCCESS){
        return YES;
    } else {
        return NO;
    }
}

//----------------------------------------------------------------------------//
#pragma mark - Life cycle Management Method
//----------------------------------------------------------------------------//
- (void)willEnterForeground
{
    qpQrCodeCapture_.delegate = self;
}

- (void)didEnterBackground
{
    [self initializeTargetInfo];
    [Utility UIButtonDisableWithColor:self.printUIButton];
    qpQrCodeCapture_.delegate = nil;
}

//----------------------------------------------------------------------------//
#pragma mark - Indicator Method
//----------------------------------------------------------------------------//
- (void)showIndicator
{
    qpIndicatorView_ = [[IndicatorView alloc] init];
    [qpIndicatorView_ show:[[UIApplication sharedApplication] keyWindow]];
    [[NSRunLoop currentRunLoop] runUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.05]];
}

- (void)hideIndicator
{
    [qpIndicatorView_ hide];
    qpIndicatorView_ = nil;
}

//----------------------------------------------------------------------------//
#pragma mark - Resize UI Method
//----------------------------------------------------------------------------//
- (void)resizeQPPreviewTargetView
{
    // Resize preview view
    CGFloat height = (_qpQrCodeScanUILable.frame.origin.y - 20) - _qpPreviewTargetUIView.frame.origin.y;
    if(height > 0){
        CGRect frame = CGRectMake(_qpPreviewTargetUIView.frame.origin.x, _qpPreviewTargetUIView.frame.origin.y,
                                  _qpPreviewTargetUIView.frame.size.width, height);
        [_qpPreviewTargetUIView setFrame:frame];
    }
}

//----------------------------------------------------------------------------//
#pragma mark - Press Button Method
//----------------------------------------------------------------------------//
- (IBAction)pressPrintUIButton:(id)sender {
    
    [qpQrCodeCapture_ stopCapture];
    [self showIndicator];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_BACKGROUND, 0), ^{
        
        // Print
        PrintSampleTask* printSampleTask = [[PrintSampleTask alloc] initWithPrinterModel:easySelectInfo_.printerName
                                                                              macAddress:easySelectInfo_.macAddress
                                                                          connectionType:easySelectInfo_.deviceType];
        
        BOOL isPrintSuccess = [printSampleTask printSample];
        dispatch_async(dispatch_get_main_queue(),  ^{
            [self hideIndicator];
            if(isPrintSuccess){
                [qpQrCodeCapture_ startCapture:self.qpPreviewTargetUIView scanDelegate:self];
            } else {
                [self showQpErrorMsg:QuickParingErrorPrint];
                // Start capturing after alert ok button clicked  (-alertView:clickedButtonAtIndex:)
            }
        });
    });
}

//----------------------------------------------------------------------------//
#pragma mark - UIAlertViewDelegate
//----------------------------------------------------------------------------//
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    // Error alert ok button clicked, start capturing
    [qpQrCodeCapture_ startCapture:self.qpPreviewTargetUIView scanDelegate:self];
}

//----------------------------------------------------------------------------//
#pragma mark - update target printer info //----------------------------------------------------------------------------//
- (void)updateTargetInfo:(EposEasySelectInfo*)easySelectInfo
{
    _targetDeviceNameUILabel.text = easySelectInfo.printerName;
    _targetInterfaceUILabel.text = [Utility interfaceString:easySelectInfo.deviceType];
    _targetMACAddressUILabel.text = easySelectInfo.macAddress;

    easySelectInfo_ = easySelectInfo;
    return;
}

//----------------------------------------------------------------------------//
#pragma mark - initialize target printer info
//----------------------------------------------------------------------------//
- (void)initializeTargetInfo
{
    _connectingUILabel.text = @"";
    _targetDeviceNameUILabel.text = @"";
    _targetInterfaceUILabel.text = @"";
    _targetMACAddressUILabel.text = @"";
    
    easySelectInfo_ = nil;

}

//----------------------------------------------------------------------------//
#pragma mark - Error Message Method
//----------------------------------------------------------------------------//
- (void)showQpErrorMsg:(QuickParingError)errorid
{
    NSString* msg = [self qrErrorMsg:errorid];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"dialog_title_error", @"")
                                                    message:msg
                                                   delegate:self
                                          cancelButtonTitle:nil
                                          otherButtonTitles:NSLocalizedString(@"dialog_button_ok", @""), nil];
    [alert show];
}

- (NSString*)qrErrorMsg:(QuickParingError)errorid
{
    switch (errorid) {
        case QuickParingErrorOpen:              return NSLocalizedString(@"QP_msg_ErrorPrinterOpen", @"");
        case QuickParingErrorPrint:             return NSLocalizedString(@"QP_msg_PrintError", @"");
        default:                                return @"";
    }
}

@end
