﻿using ePOS_Print_CS.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using Windows.Storage;
using System.Threading.Tasks;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.Graphics.Imaging;
using Windows.UI.Xaml.Media.Imaging;
using Windows.ApplicationModel.Resources;

using LibEposPrint;

namespace ePOS_Print_CS
{
    public sealed partial class ImagePage : Page
    {
        const int SEND_TIMEOUT = 10 * 1000;
        const int IMAGE_WIDTH_MAX = 512;
        const int COLOR_MONO = 0;
        const int COLOR_GRAY = 1;
        const int HALFTONE_DITHER = 0;
        const int HALFTON_ERROR_DIFF = 1;
        const int HALFTON_THRESHOLD = 2;

        private ShowMsg showMsg = null;
        private ResourceLoader loader = null;
        private BitmapDecoder m_selectImage = null;

        private NavigationHelper navigationHelper;
        public NavigationHelper NavigationHelper
        {
            get { return this.navigationHelper; }
        }

        public ImagePage()
        {
            this.InitializeComponent();
            this.navigationHelper = new NavigationHelper(this);

            showMsg = new ShowMsg();
            loader = new ResourceLoader();

            StatusChangeEvent statusChangeEvent = new StatusChangeEvent(Status_Change_Event);
            BatteryStatusChangeEvent batteryStatusChangevent = new BatteryStatusChangeEvent(Battery_Status_Change_Event);

            Print printer = MainPage.getPrinter();
            if (printer == null)
            {
                return;
            }
            else
            {
                printer.SetStatusChangeEventCallback(statusChangeEvent);
                printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent);
            }

            //init Color Mode list
            this.ColorMode_ComboBox.Items.Add(loader.GetString("mode_mono"));
            this.ColorMode_ComboBox.Items.Add(loader.GetString("mode_gray16"));

            //init halftone list
            this.HlaftoneMethod_ComboBox.Items.Add(loader.GetString("halftone_dither"));
            this.HlaftoneMethod_ComboBox.Items.Add(loader.GetString("halftone_error_diffusion"));
            this.HlaftoneMethod_ComboBox.Items.Add(loader.GetString("halftone_threshold"));

            //default setting
            this.ColorMode_ComboBox.SelectedIndex = COLOR_MONO;
            this.HlaftoneMethod_ComboBox.SelectedIndex = HALFTONE_DITHER;

            this.Brightness_TxtBox.Text = "1.0";
        }

        private async void SelectImage_Button_Click(object sender, RoutedEventArgs e)
        {
            IRandomAccessStreamWithContentType selectStream = null;

            try
            {
                FileOpenPicker picker = new FileOpenPicker();

                picker.FileTypeFilter.Add(".jpeg");
                picker.FileTypeFilter.Add(".png");
                picker.FileTypeFilter.Add(".gif");
                picker.FileTypeFilter.Add(".tiff");
                picker.FileTypeFilter.Add(".bmp");
                picker.FileTypeFilter.Add(".ico");
                picker.FileTypeFilter.Add(".jxr");

                picker.ViewMode = PickerViewMode.Thumbnail;
                picker.SuggestedStartLocation = PickerLocationId.Desktop;
                StorageFile imageFile = await picker.PickSingleFileAsync();

                if (imageFile == null)
                {
                    return;
                }

                selectStream = await imageFile.OpenReadAsync();

                BitmapImage image = new BitmapImage();
                image.SetSource(selectStream);

                viewImageArea.Source = image;

                m_selectImage = await BitmapDecoder.CreateAsync(selectStream);
            }
            catch(Exception)
            {
                m_selectImage = null;
            }
        }

        private async void Print_Button_Click(object sender, RoutedEventArgs e)
        {
            this.IsEnabled = false;
            await printImage();
            this.IsEnabled = true;
        }

        private async Task printImage()
        {
            if (m_selectImage == null)
            {
                showMsg.showError(loader.GetString("errmsg_noimage"));
                return;
            }

            Builder builder = null;
            Print printer = null;
            PrinterStatus sendStatus;
            sendStatus.printerStatus = 0;
            sendStatus.batteryStatus = 0;
            string method = "";

            try
            {
                //create builder
                method = "Builder";
                builder = new Builder(MainPage.getPrinterName(), MainPage.getLanguage());

                //add command
                method = "AddImageAsync";
                await builder.AddImageAsync(m_selectImage, 0, 0, (int)Math.Min(IMAGE_WIDTH_MAX, m_selectImage.PixelWidth),
                                            (int)m_selectImage.PixelHeight, Color.COLOR_1, getBuilderMode(), getBuilderHalftone(), getBuilderBrightness());

                //send builder data
                try
                {
                    printer = MainPage.getPrinter();
                    sendStatus = await printer.SendDataAsync(builder, SEND_TIMEOUT);
                    showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus);
                }
                catch (Exception ex)
                {
                    PrinterStatus status = printer.GetPrinterStatus(ex.HResult);
                    showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus);
                }
            }
            catch (Exception ex)
            {
                showMsg.showEposException(ex.HResult, method);
            }

            //remove builder
            if(builder !=null)
            {
                try
                {
                    builder.ClearCommandBuffer();
                    builder = null;
                }
                catch (Exception)
                {
                    builder = null;
                }
            }
        }

        private Mode getBuilderMode()
        {
            switch(ColorMode_ComboBox.SelectedIndex)
            {
                case COLOR_GRAY:
                    return Mode.MODE_GRAY16;
                case COLOR_MONO:
                default:
                    return Mode.MODE_MONO;
            }
        }

        private Halftone getBuilderHalftone()
        {
            switch(HlaftoneMethod_ComboBox.SelectedIndex)
            {
                case HALFTON_ERROR_DIFF:
                    return Halftone.HALFTONE_ERROR_DIFFUSION;
                case HALFTON_THRESHOLD:
                    return Halftone.HALFTONE_THRESHOLD;
                case HALFTONE_DITHER:
                default:
                    return Halftone.HALFTONE_DITHER;
            }
        }

        private double getBuilderBrightness()
        {
            try
            {
                return double.Parse(Brightness_TxtBox.Text);
            }
            catch(Exception)
            {
                return 1.0;
            }
        }

        private void Status_Change_Event(string deviceName, int status)
        {
            showMsg.showStatusChangeEvent(deviceName, status);
        }

        private void Battery_Status_Change_Event(string deviceName, int battery)
        {
            showMsg.showBatteryStatusChangeEvent(deviceName, battery);
        }
    }
}
