﻿Imports System.Threading.Tasks
Imports Windows.Storage.Streams
Imports Windows.ApplicationModel.Resources
Imports LibEposPrint

Public NotInheritable Class GetPrinterNamePage
    Inherits Page

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing

    Const RESPONSE_HEADER = &H5F
    Const RESPONSE_TERMINAL = &H0
    Const RESPONSE_TIMEOUT = 1000
    Const RESPONSE_MAXBYTE = 128

    Dim fontName As String = Nothing
    Dim printerName As String = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        ' select default radiobutton
        Select Case (MainPage.GetDeviceType())
            Case DevType.DEVTYPE_BLUETOOTH
                Me.BT_RadioButton.IsChecked = True
            Case Else
                Me.TCP_RadioButton.IsChecked = True
        End Select

        ' init edit
        Me.DeviceName_TxtBox.Text = MainPage.GetDeviceName()

    End Sub

    Private Async Sub GetPrinterName_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await GetPrinterName()
        Me.IsEnabled = True
    End Sub

    Private Async Function GetPrinterName() As Task

        If String.IsNullOrWhiteSpace(DeviceName_TxtBox.Text) Then
            _showMsg.showError(_loader.GetString("errmsg_notext"))
            Return
        End If

        Dim CMD_ESDPOS As Byte() = {&H1B, &H3D, &H1,
                             &H1D, &H49, &H45,
                             &H1D, &H49, &H43}

        Dim CMD_ESC_ENABLE_PRINTER As Byte() = {&H1B, &H3D, &H1}

        Dim CMD_GS_I_PRINTER_NAME As Byte() = {&H1D, &H49, &H43}

        Dim CMD_GS_I_ADDITIONAL_FONTS As Byte() = {&H1D, &H49, &H45}

        Dim deviceType As IoDevType = IoDevType.TCP

        If BT_RadioButton.IsChecked Then
            deviceType = IoDevType.BLUETOOTH
        Else
            deviceType = IoDevType.TCP
        End If

        fontName = Nothing
        printerName = Nothing

        Dim port As EpsonIo = Nothing
        Dim method As String = String.Empty

        Try
            port = New EpsonIo()
            Dim receiveBuffer As Byte() = New Byte(RESPONSE_MAXBYTE) {}
            Dim ret As Boolean = False

            'open
            port = New EpsonIo()
            method = "OpenAsync"
            Await port.OpenAsync(deviceType, DeviceName_TxtBox.Text, "")

            Await clearReceiveBuffer(port)

            'send command(esc/pose)
            'enable printer
            method = "Write"
            port.WriteBytes(CMD_ESC_ENABLE_PRINTER, 0, CMD_ESC_ENABLE_PRINTER.Length)
            Await port.WriteAsync(RESPONSE_TIMEOUT)

            'printer name
            port.WriteBytes(CMD_GS_I_PRINTER_NAME, 0, CMD_GS_I_PRINTER_NAME.Length)
            Await port.WriteAsync(RESPONSE_TIMEOUT)

            method = "Read"
            ret = Await ReceiveResponse(port, receiveBuffer)

            If ret Then
                'additonal fonts
                method = "Write"
                port.WriteBytes(CMD_GS_I_ADDITIONAL_FONTS, 0, CMD_GS_I_ADDITIONAL_FONTS.Length)
                Await port.WriteAsync(RESPONSE_TIMEOUT)

                method = "Read"
                ret = Await ReceiveResponse(port, receiveBuffer)
            End If

            'close
            method = "closeAsync"
            Await port.CloseAsync()
        Catch ex As Exception
            _showMsg.showIoException(ex.HResult, method)
            ' cannot await
            GoTo ERR
        End Try

        If fontName Is Nothing Or printerName Is Nothing Then
            _showMsg.showError(_loader.GetString("errmsg_getname"))
        Else
            _showMsg.showPrinterName(printerName, getLanguageSpec(fontName))
        End If
        Return
ERR:
        If port IsNot Nothing Then
            Try
                Await port.CloseAsync()
                port = Nothing
            Catch ex As Exception
                port = Nothing
            End Try
        End If

    End Function

    Private Async Function clearReceiveBuffer(port As EpsonIo) As Task
        Try
            Dim receiveData As Byte() = New Byte(RESPONSE_MAXBYTE) {}
            Await port.ReadAsync(RESPONSE_MAXBYTE, RESPONSE_TIMEOUT)
            port.ReadBytes(receiveData, receiveData, 0, RESPONSE_MAXBYTE)
        Catch ex As Exception
            Throw ex
        End Try
    End Function

    Private Async Function ReceiveResponse(port As EpsonIo, receiveBuffer As Byte()) As Task(Of Boolean)
        If (receiveBuffer Is Nothing Or 0 >= receiveBuffer.Length) Then
            Return False
        End If

        Dim readSize As Integer = 0

        'receive
        Try
            Await port.ReadAsync(receiveBuffer.Length, RESPONSE_TIMEOUT)
            readSize = port.ReadBytes(receiveBuffer, receiveBuffer, 0, receiveBuffer.Length)
        Catch ex As Exception
            Throw ex
        End Try

        If 0 < readSize Then
            Dim response As Byte() = New Byte(receiveBuffer.Length) {}
            receiveBuffer.CopyTo(response, 0)

            If Not AnalyzeResponse(response) Then
                Return False
            End If
        End If

        Return True

    End Function

    Private Function AnalyzeResponse(response As Byte()) As Boolean

        Dim currentPos As Integer = 0

        'search 5f header
        For currentPos = 0 To response.Length - 1
            If response(currentPos) = RESPONSE_HEADER Then
                currentPos += 1
                Exit For
            End If
        Next

        If (currentPos >= response.Length) Then
            Return False
        End If

        ' terminate check
        Dim endPos As Integer = 0
        For endPos = currentPos To response.Length - 1
            If response(endPos) = RESPONSE_TERMINAL Then
                Exit For
            End If
        Next

        'get Response string
        Dim responseString As String = Nothing
        Try
            responseString = System.Text.Encoding.GetEncoding("US-ASCII").GetString(response, currentPos, endPos - currentPos)
        Catch ex As Exception
            Return False
        End Try

        If printerName Is Nothing Then
            printerName = responseString.TrimEnd()
        Else
            fontName = responseString.TrimEnd()
        End If

        Return True

    End Function

    Private Function getLanguageSpec(font As String) As String

        If ("" = font) Then
            Return _loader.GetString("language_ank")
        ElseIf ("KANJI JAPANESE" = font) Then
            Return _loader.GetString("language_japanese")
        Else
            Return _loader.GetString("language_other")
        End If

    End Function
End Class
