﻿Imports Windows.Storage
Imports System.Threading.Tasks
Imports Windows.Storage.Pickers
Imports Windows.Storage.Streams
Imports Windows.Graphics.Imaging
Imports Windows.UI.Xaml.Media.Imaging
Imports Windows.ApplicationModel.Resources

Imports LibEposPrint

Public NotInheritable Class ImagePage
    Inherits Page

    Const SEND_TIMEOUT = 10 * 1000
    Const IMAGE_WIDTH_MAX = 512
    Const COLOR_MONO = 0
    Const COLOR_GRAY = 1
    Const HALFTONE_DITHER = 0
    Const HALFTON_ERROR_DIFF = 1
    Const HALFTON_THRESHOLD = 2

    Private _showMsg As ShowMsg = Nothing
    Private _loader As ResourceLoader = Nothing
    Private _selectImage As BitmapDecoder = Nothing

    Private _navigationHelper As Common.NavigationHelper
    Public ReadOnly Property NavigationHelper As Common.NavigationHelper
        Get
            Return Me._navigationHelper
        End Get
    End Property

    Public Sub New()

        InitializeComponent()
        Me._navigationHelper = New Common.NavigationHelper(Me)

        _showMsg = New ShowMsg()
        _loader = New ResourceLoader()

        Dim statusChangeEvent As StatusChangeEvent = New StatusChangeEvent(AddressOf Me.Status_Change_Event)
        Dim batteryStatusChangevent As BatteryStatusChangeEvent = New BatteryStatusChangeEvent(AddressOf Me.Battery_Status_Change_Event)

        Dim printer As Print = MainPage.GetPrinter()

        If printer Is Nothing Then
            Return
        Else
            printer.SetStatusChangeEventCallback(statusChangeEvent)
            printer.SetBatteryStatusChangeEventCallback(batteryStatusChangevent)
        End If

        ' init Color Mode list
        Me.ColorMode_ComboBox.Items.Add(_loader.GetString("mode_mono"))
        Me.ColorMode_ComboBox.Items.Add(_loader.GetString("mode_gray16"))

        ' init halftone list
        Me.HlaftoneMethod_ComboBox.Items.Add(_loader.GetString("halftone_dither"))
        Me.HlaftoneMethod_ComboBox.Items.Add(_loader.GetString("halftone_error_diffusion"))
        Me.HlaftoneMethod_ComboBox.Items.Add(_loader.GetString("halftone_threshold"))

        ' Default setting
        Me.ColorMode_ComboBox.SelectedIndex = 0
        Me.HlaftoneMethod_ComboBox.SelectedIndex = 0

        Me.Brightness_TxtBox.Text = "1.0"

    End Sub

    Private Async Sub SelectImage_Button_Click(sender As Object, e As RoutedEventArgs)

        Dim selectStream As IRandomAccessStreamWithContentType = Nothing

        Try
            Dim picker As FileOpenPicker = New FileOpenPicker()

            picker.FileTypeFilter.Add(".jpeg")
            picker.FileTypeFilter.Add(".png")
            picker.FileTypeFilter.Add(".gif")
            picker.FileTypeFilter.Add(".tiff")
            picker.FileTypeFilter.Add(".bmp")
            picker.FileTypeFilter.Add(".ico")
            picker.FileTypeFilter.Add(".jxr")

            picker.ViewMode = PickerViewMode.Thumbnail
            picker.SuggestedStartLocation = PickerLocationId.Desktop
            Dim imageFile As StorageFile = Await picker.PickSingleFileAsync()

            If imageFile Is Nothing Then
                Return
            End If

            selectStream = Await imageFile.OpenReadAsync()

            Dim image As BitmapImage = New BitmapImage()
            image.SetSource(selectStream)

            viewImageArea.Source = image

            _selectImage = Await BitmapDecoder.CreateAsync(selectStream)
        Catch ex As Exception
            _selectImage = Nothing
        End Try

    End Sub

    Private Async Sub Print_Button_Click(sender As Object, e As RoutedEventArgs)
        Me.IsEnabled = False
        Await PrintImage()
        Me.IsEnabled = True
    End Sub

    Private Async Function PrintImage() As Task

        If _selectImage Is Nothing Then
            _showMsg.showError(_loader.GetString("errmsg_noimage"))
            Return
        End If



        Dim builder As Builder = Nothing
        Dim printer As Print = Nothing
        Dim sendStatus As PrinterStatus
        sendStatus.printerStatus = 0
        sendStatus.batteryStatus = 0
        Dim method As String = String.Empty

        Try
            ' create builder
            method = "Builder"
            builder = New Builder(MainPage.GetPrinterName(), MainPage.GetLanguage())

            ' add command
            method = "AddImageAsync"
            Await builder.AddImageAsync(_selectImage, 0, 0, Math.Min(IMAGE_WIDTH_MAX, _selectImage.PixelWidth),
                                        _selectImage.PixelHeight, Color.COLOR_1, GetBuilderMode(), GetBuilderHalftone(), GetBuilderBrightness())

            ' send builder data
            Try
                printer = MainPage.GetPrinter()
                sendStatus = Await printer.SendDataAsync(builder, SEND_TIMEOUT)
                _showMsg.showStatus(EposHResult.HR_S_OK, sendStatus.printerStatus, sendStatus.batteryStatus)
            Catch ex As Exception
                Dim status As PrinterStatus = printer.GetPrinterStatus(ex.HResult)
                _showMsg.showStatus(ex.HResult, status.printerStatus, status.batteryStatus)
            End Try
        Catch ex As Exception
            _showMsg.showEposException(ex.HResult, method)
        End Try

        ' remove builder
        If builder IsNot Nothing Then
            Try
                builder.ClearCommandBuffer()
                builder = Nothing
            Catch ex As Exception
                builder = Nothing
            End Try
        End If

    End Function

    Private Function GetBuilderMode() As Mode

        Select Case (ColorMode_ComboBox.SelectedIndex)
            Case COLOR_GRAY
                Return Mode.MODE_GRAY16
            Case Else
                Return Mode.MODE_MONO
        End Select

    End Function

    Private Function GetBuilderHalftone() As Halftone

        Select Case (HlaftoneMethod_ComboBox.SelectedIndex)
            Case HALFTON_ERROR_DIFF
                Return Halftone.HALFTONE_ERROR_DIFFUSION
            Case HALFTON_THRESHOLD
                Return Halftone.HALFTONE_THRESHOLD
            Case Else
                Return Halftone.HALFTONE_DITHER
        End Select

    End Function

    Private Function GetBuilderBrightness() As Double

        Try
            Return Double.Parse(Brightness_TxtBox.Text)
        Catch
            Return 1.0
        End Try

    End Function

    Private Sub Status_Change_Event(deviceName As String, status As Integer)
        _showMsg.showStatusChangeEvent(deviceName, status)
    End Sub

    Private Sub Battery_Status_Change_Event(deviceName As String, battery As Integer)
        _showMsg.showBatteryStatusChangeEvent(deviceName, battery)
    End Sub

End Class
