#import "GetNameView.h"
#import "ShowMsg.h"
#import "ePOS-Print.h"
#import <sys/time.h>

static unsigned char CMD_ESDPOS[] = {
    0x1b, 0x3d, 0x01,    // ESC = 1(Enables printer)
    0x1d, 0x49, 0x45,    // GS I 69(Type of mounted additional fonts)
    0x1d, 0x49, 0x43,    // GS I 67(Printer name)
};
#define RESPONSE_HEADER       0x5f
#define RESPONSE_TERMINAL     0x00
#define SEND_RESPONSE_TIMEOUT 1000
#define RESPONSE_MAXBYTE      128

@interface GetNameView ()
- (void)getPrinterName;
- (long long)currentTimeMillis;
- (int)receiveResponse:(EpsonIo*)port;
- (BOOL)analyzeResponse:(NSData*)receiveData length:(int)bufferlen;
- (NSString*)getLanguageSpec:(NSString*)font;
@end

@implementation GetNameView
@synthesize textIp_;
@synthesize scrollView_;
@synthesize itemView_;
@synthesize deviceType_;

- (id)initWithConnectionType:(int)connectionType address:(NSString*)address {
    self = [super init];
    if (self) {
        initIpAddress_ = [address retain];
        initConnectionType_ = connectionType;
    }
    return self;
    
}

- (void)dealloc
{
    [textIp_ release];
    [scrollView_ release];
    [itemView_ release];

    [initIpAddress_ release];
    [fontName_ release];
    [printerName_ release];
    [deviceType_ release];
    [super dealloc];
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //init device type
    switch(initConnectionType_){
        case EPOS_OC_DEVTYPE_TCP:
            deviceType_.selectedSegmentIndex = 0;
            break;
        default:
            deviceType_.selectedSegmentIndex = 1;
            break;
    }

    //init edit
    textIp_.text = initIpAddress_;
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    //auto scroll setting
    scrollBaseView_ = scrollView_;
    [scrollView_ setContentSize:itemView_.frame.size];
    [self registerForKeyboardNotifications];
}

- (void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setTextIp_:nil];
    [self setScrollView_:nil];
    [self setItemView_:nil];
    [self setDeviceType_:nil];
    [super viewDidUnload];
}

- (IBAction)eventDidPush:(id)sender {
    [self getPrinterName];
}

- (void)getPrinterName
{
    //get open parameter
    if(textIp_.text.length == 0){
        [ShowMsg showError:@"errmsg_noaddress"];
        return ;
    }
    
    //open device type
    int connectionType;
    switch(deviceType_.selectedSegmentIndex){
        case 0:
            connectionType = EPSONIO_OC_DEVTYPE_TCP;
            break;
        default:
            connectionType = EPSONIO_OC_DEVTYPE_BLUETOOTH;
            break;
    }    
    
    //open
    EpsonIo *port = [[EpsonIo alloc] init];
    if(port == nil){
        return ;
    }
    int result = [port open:connectionType DeviceName:textIp_.text DeviceSettings:nil];
    if(result != EPSONIO_OC_SUCCESS){
        [ShowMsg showExceptionEpsonIo:result method:@"open"];
        [port release];
        return;
    }
        
    //send command(esc/pos)
    size_t written = 0;
    NSData * escpos = [[[NSData alloc] initWithBytes:CMD_ESDPOS length:sizeof(CMD_ESDPOS)] autorelease];
    result = [port write:escpos Offset:0 Size:[escpos length] Timeout:SEND_RESPONSE_TIMEOUT SizeWritten:&written];
    if(result != EPSONIO_OC_SUCCESS){
        [ShowMsg showExceptionEpsonIo:result method:@"write"];
        [port close];
        [port release];
        return;
    }

    //receive response
    result = [self receiveResponse:port];
    if(result != EPSONIO_OC_SUCCESS){
        [ShowMsg showExceptionEpsonIo:result method:@"read"];
        [port close];
        [port release];
        return;
    }
       
    //close
    [port close];
    [port release];
    
    if(fontName_ == nil || printerName_ == nil){
        [ShowMsg showError:@"errmsg_getname"];
    }else{
        [ShowMsg showPrinterName:printerName_ languageName:[self getLanguageSpec:fontName_]];
    }
         
    [fontName_ release];
    [printerName_ release];
    fontName_ = nil;
    printerName_ = nil;
}

- (long long)currentTimeMillis
{
    struct timeval ts;
    gettimeofday(&ts, NULL);
    return (unsigned long long)ts.tv_sec * 1000 + ts.tv_usec / 1000;
}

//receive response
- (int)receiveResponse:(EpsonIo*)port
{
    NSMutableData * receiveBuffer = [[[NSMutableData alloc] initWithLength:RESPONSE_MAXBYTE] autorelease];
    int receiveCurrent = 0;
    
    //receive loop(timeout=1000ms)
    long long starttime = [self currentTimeMillis];
    while(true){
        //check receive buffer full
        if(receiveCurrent >= [receiveBuffer length]){
            break;
        }
        
        //timeout check
        long long readTimeout = SEND_RESPONSE_TIMEOUT - ([self currentTimeMillis] - starttime);
        if(readTimeout < 0){
            break;
        }
        
        //receive
        size_t sizeRead;
        int result = [port read:receiveBuffer
                         Offset:receiveCurrent
                           Size:receiveBuffer.length - receiveCurrent
                        Timeout:readTimeout
                       SizeRead:&sizeRead];
        if(result == EPSONIO_OC_ERR_TIMEOUT){
            break;
        }else if(result != EPSONIO_OC_SUCCESS){
            return result;
        }
        receiveCurrent += sizeRead;
        
        //analyze receive data
        if([self analyzeResponse:receiveBuffer length:receiveCurrent]){
            break;
        }
    }
    
    return EPSONIO_OC_SUCCESS;
}

//analyze GS I response
- (BOOL)analyzeResponse:(NSData*)receiveData length:(int)bufferlen
{
    int currentPos = 0;
    NSString* responseFontName = nil;
    NSString* responsePrinterName = nil;
    
    const unsigned char *buffer = [receiveData bytes];
    //search 5f header
    for(; currentPos < bufferlen; currentPos++){
        if(buffer[currentPos] == RESPONSE_HEADER){
            break;
        }
    }
    if(currentPos == bufferlen){
        return false;
    }
    while(responseFontName == nil || responsePrinterName == nil){
        //check 5f header
        if(buffer[currentPos] != RESPONSE_HEADER){
            return false;
        }
        
        //find terminal(0x00)
        int endPos = currentPos + 1;
        for(; endPos < bufferlen; endPos++){
            if(buffer[endPos] == RESPONSE_TERMINAL){
                break;
            }
        }
        if(endPos >= bufferlen){
            return false;
        }
        
        //get response string
        NSString *responseString
            = [[[NSString alloc] initWithBytes:buffer + currentPos + 1
                                       length:endPos - currentPos - 1
                                     encoding:NSASCIIStringEncoding] autorelease];
        if(responseFontName == nil){
            responseFontName = responseString;
        }else{
            responsePrinterName = responseString;
        }
        
        currentPos = endPos + 1;
    }
    
    fontName_ = [[responseFontName stringByTrimmingCharactersInSet:
                  [NSCharacterSet whitespaceCharacterSet]] retain];
    printerName_ = [[responsePrinterName stringByTrimmingCharactersInSet:
                  [NSCharacterSet whitespaceCharacterSet]] retain];
    return true;
}

- (NSString*)getLanguageSpec:(NSString*)font
{
    if([font isEqualToString:@""]){
        return NSLocalizedString(@"language_ank", @"");
    }else if([font isEqualToString:@"KANJI JAPANESE"]){
        return NSLocalizedString(@"language_japanese", @"");
   }else{
       return NSLocalizedString(@"language_other", @"");
   }
}

@end
