#import "ImageView.h"
#import "ShowMsg.h"
#import "ePOS-Print.h"

#define SEND_TIMEOUT     10 * 1000
#define IMAGE_WIDTH_MAX  512

@interface ImageView ()
- (void)selectImage;
- (void)printImage;
- (int)getBuilderMode;
- (int)getBuilderHalftone;
- (double)getBuilderBrightness;
@end

@implementation ImageView
@synthesize imageView_;
@synthesize scrollView_;
@synthesize buttonMode_;
@synthesize buttonHalftone_;
@synthesize textBrightness_;

- (id)initWithPrinter:(EposPrint*)printer
          printername:(NSString*)printername
             language:(int)language
{
    self = [super init];
    if (self) {
        printer_ = [printer retain];
        printername_ = [printername retain];
        language_ = language;
    }
    return self;
}

- (void)dealloc {
    [imageView_ release];
    [printer_ release];
    [printername_ release];
    [selectImage_ release];
    [modeList_ release];
    [halftoneList_ release];
    [textBrightness_ release];
    [super dealloc];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    imageView_.contentMode = UIViewContentModeScaleAspectFit;
    
    //init mode list
    modeList_ = [[PickerTableView alloc] init];
    NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"mode_mono", @"")];
    [items addObject:NSLocalizedString(@"mode_gray16", @"")];
    [buttonMode_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [modeList_ setItemList:items];
    modeList_.delegate = self;
    
    //init halftone list
    halftoneList_ = [[PickerTableView alloc] init];
    items = [[[NSMutableArray alloc] init] autorelease];
    [items addObject:NSLocalizedString(@"halftone_dither", @"")];
    [items addObject:NSLocalizedString(@"halftone_error_diffusion", @"")];
    [items addObject:NSLocalizedString(@"halftone_threshold", @"")];
    [buttonHalftone_ setTitle:[items objectAtIndex:0] forState:UIControlStateNormal];
    [halftoneList_ setItemList:items];
    halftoneList_.delegate = self;
    
    //default setting
    textBrightness_.text = @"1.0";
    
    //show back button(navigationbar)
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    
    scrollBaseView_ = scrollView_;
    [scrollView_ setContentSize:CGSizeMake(320, 685)];
    [self registerForKeyboardNotifications];
}

- (void)viewDidUnload
{
    [self unregisterForKeyboardNotifications];
    [self setImageView_:nil];
    [self setButtonMode_:nil];
    [self setButtonHalftone_:nil];
    [self setTextBrightness_:nil];
    [super viewDidUnload];
}

- (void)onSelectPickerItem:(NSInteger)position obj:(id)obj {
    if(obj == modeList_){
        [buttonMode_ setTitle:[modeList_ getItem:position] forState:UIControlStateNormal];
    }else if(obj == halftoneList_){
        [buttonHalftone_ setTitle:[halftoneList_ getItem:position] forState:UIControlStateNormal];
    }
}

- (IBAction)eventDidPush:(id)sender {
    switch(((UIView*)sender).tag){
        case 0:
            [self printImage];
            break;
        case 1:
            [self selectImage];
            break;
        case 2:
            [modeList_ show];
            break;
        case 3:
            [halftoneList_ show];
            break;
        default:
            break;
    }
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info {
    [self dismissModalViewControllerAnimated:YES];
    
    imageView_.image = nil;
    [selectImage_ release];
    selectImage_ = nil;
    
    selectImage_ =[[info objectForKey:UIImagePickerControllerOriginalImage] retain];
    imageView_.image = selectImage_;
}

- (void)selectImage
{
    UIImagePickerControllerSourceType sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    if([UIImagePickerController isSourceTypeAvailable:sourceType]){
        UIImagePickerController *picker = [[[UIImagePickerController alloc] init] autorelease];
        picker.delegate = self;
        picker.sourceType = sourceType;
        [self presentModalViewController:picker animated:YES];
    }
}

- (void)printImage
{
    if(selectImage_ == nil){
        [ShowMsg showError:@"errmsg_noimage"];
        return ;
    }
    
    //create builder
    EposBuilder *builder = [[[EposBuilder alloc] initWithPrinterModel:printername_ Lang:language_] autorelease];
    if(builder == nil){
        return ;
    }
    
    //add command
    int result = [builder addImage:selectImage_ X:0 Y:0 Width:MIN(IMAGE_WIDTH_MAX, selectImage_.size.width) Height:selectImage_.size.height Color:EPOS_OC_COLOR_1 Mode:[self getBuilderMode] Halftone:[self getBuilderHalftone] Brightness:[self getBuilderBrightness]];
    if(result != EPOS_OC_SUCCESS){
        [ShowMsg showExceptionEpos:result method:@"addImage"];
        return ;
    }

    //send builder data
    unsigned long status = 0;
    unsigned long battery = 0;
    result = [printer_ sendData:builder Timeout:SEND_TIMEOUT Status:&status Battery:&battery];
    [ShowMsg showStatus:result status:status battery:battery];
    
    //remove builder
    [builder clearCommandBuffer];
}

- (int)getBuilderMode
{
    switch(modeList_.selectIndex){
        case 1:
            return EPOS_OC_MODE_GRAY16;
        case 0:
        default:
            return EPOS_OC_MODE_MONO;
    }
}

- (int)getBuilderHalftone
{
    switch(halftoneList_.selectIndex){
        case 1:
            return EPOS_OC_HALFTONE_ERROR_DIFFUSION;
        case 2:
            return EPOS_OC_HALFTONE_THRESHOLD;
        case 0:
        default:
            return EPOS_OC_HALFTONE_DITHER;
    }
}

- (double)getBuilderBrightness
{
    return [textBrightness_.text doubleValue];
}

@end
